@@ -74,11 +74,16 @@ class ResultStorage:
         self,
         technology: Optional[str] = None,
         version: Optional[str] = None,
+        version_min: Optional[str] = None,
+        version_max: Optional[str] = None,
         technology_type: Optional[str] = None,
         probability: Optional[int] = 100,
         vulnerability: Optional[str] = None,
         description: Optional[str] = None,
-        module: Optional[str] = None
+        module: Optional[str] = None,
+        product_id: Optional[int] = None,
+        vendor: Optional[str] = None,
+        cve_details_url: Optional[str] = None
     ) -> None:
         """
         Add a record to the storage. Automatically detects the calling module if `module` is not provided.
@@ -86,11 +91,16 @@ class ResultStorage:
         Args:
             technology: Technology name (e.g., OS, WebServer).
             version: Version of the technology.
+            version_min: Minimum version in range.
+            version_max: Maximum version in range.
             technology_type: Type of the technology (e.g., 'webserver', 'os').
             probability: Probability value (0-100).
             vulnerability: Vulnerability identifier.
             description: Description or additional information.
             module: Name of the calling module (optional; autodetected if not provided).
+            product_id: Product ID from products.json.
+            vendor: Vendor name.
+            cve_details_url: URL to CVE details page.
 
         Returns:
             None
@@ -104,10 +114,15 @@ class ResultStorage:
             "module": (module or "").strip().upper(),
             "technology": (technology or "").strip(),
             "version": (version or "").strip() if version else None,
+            "version_min": (version_min or "").strip() if version_min else None,
+            "version_max": (version_max or "").strip() if version_max else None,
             "technology_type": (technology_type or "").strip() if technology_type else None,
             "probability": probability,
             "vulnerability": (vulnerability or "").strip() if vulnerability else None,
-            "description": (description or "").strip() if description else None
+            "description": (description or "").strip() if description else None,
+            "product_id": product_id,
+            "vendor": (vendor or "").strip() if vendor else None,
+            "cve_details_url": (cve_details_url or "").strip() if cve_details_url else None
         }
 
         with self._lock:
@@ -150,11 +165,16 @@ class ResultStorage:
         """
         with self._lock:
             unique_pairs = {
-                (record.get("technology"), record.get("version"))
+                (
+                    record.get("technology"), 
+                    record.get("version"),
+                    record.get("version_min"),
+                    record.get("version_max")
+                )
                 for record in self._storage
                 if record.get("technology") # skip empty technologies
             }
-        return [{"technology": tech, "version": ver} for tech, ver in unique_pairs]
+        return [{"technology": tech, "version": ver, "version_min": ver_min, "version_max": ver_max} for tech, ver, ver_min, ver_max in unique_pairs]
 
     def get_data_for_technology(self, technology: str, version: Optional[str] = None) -> dict:
         """
@@ -181,18 +201,48 @@ class ResultStorage:
         with self._lock:
             filtered = [
                 r for r in self._storage
-                if r.get("technology") == technology and (version is None or r.get("version") == version)
+                if r.get("technology") == technology
             ]
             if not filtered:
                 return {}
 
-            if version is None:
-                versions_set = {r.get("version") for r in filtered}
-                non_none_versions = [v for v in versions_set if v is not None]
-                sorted_versions = sorted(non_none_versions)
-                versions = ([None] if None in versions_set else []) + sorted_versions
+            if version is not None:
+                version_filtered = [r for r in filtered if r.get("version") == version]
+                if version_filtered:
+                    filtered = version_filtered
+
+            # Prioritize version records
+            exact_records = [r for r in filtered if r.get("version")]
+            range_records = [r for r in filtered if not r.get("version") and r.get("version_min") and r.get("version_max")]
+            minimum_records = [r for r in filtered if not r.get("version") and r.get("version_min") and not r.get("version_max")]
+            maximum_records = [r for r in filtered if not r.get("version") and not r.get("version_min") and r.get("version_max")]
+            
+            # Select primary record for version display
+            if exact_records:
+                primary = exact_records[0]
+                display_version = primary.get("version")
+                display_version_min = None
+                display_version_max = None
+            elif range_records:
+                primary = range_records[0]
+                display_version = None
+                display_version_min = primary.get("version_min")
+                display_version_max = primary.get("version_max")
+            elif minimum_records:
+                primary = minimum_records[0]
+                display_version = None
+                display_version_min = primary.get("version_min")
+                display_version_max = None
+            elif maximum_records:
+                primary = maximum_records[0]
+                display_version = None
+                display_version_min = None
+                display_version_max = primary.get("version_max")
             else:
-                versions = [version]
+                primary = filtered[0]
+                display_version = None
+                display_version_min = None
+                display_version_max = None
 
             modules = sorted({r["module"] for r in filtered if r.get("module")})
             probability_sum = sum(r.get("probability") or 0 for r in filtered)
@@ -205,9 +255,16 @@ class ResultStorage:
             sw_prefix = mapping.get("swPrefix")
             sw_value = f"{sw_prefix}{technology}" if sw_prefix else None
 
+            # Get product_id, vendor, and cve_details_url from first record that has them
+            product_id = next((r.get("product_id") for r in filtered if r.get("product_id")), None)
+            vendor = next((r.get("vendor") for r in filtered if r.get("vendor")), None)
+            cve_details_url = next((r.get("cve_details_url") for r in filtered if r.get("cve_details_url")), None)
+
             return {
                 "technology": technology,
-                "versions": versions,
+                "version": display_version,
+                "version_min": display_version_min,
+                "version_max": display_version_max,
                 "count": len(filtered),
                 "modules": modules,
                 "probability_sum": probability_sum,
@@ -215,7 +272,10 @@ class ResultStorage:
                 "descriptions": descriptions,
                 "node_target_type": node_target_type,
                 "sw_type": sw_type,
-                "sw_value": sw_value
+                "sw_value": sw_value,
+                "product_id": product_id,
+                "vendor": vendor,
+                "cve_details_url": cve_details_url
             }
 
     def get_properties(self) -> dict[str, dict[str, str]]:
