@@ -23,6 +23,7 @@ import ssl
 from urllib.parse import urlparse
 from helpers.result_storage import storage
 from helpers.stored_responses import StoredResponses
+from helpers.products import get_product_manager
 
 from typing import List, Dict, Any, Optional, Tuple
 from ptlibs.ptprinthelper import ptprint
@@ -37,6 +38,7 @@ class DEFPAGE:
         self.ptjsonlib = ptjsonlib
         self.helpers = helpers
         self.http_client = http_client
+        self.product_manager = get_product_manager()
 
         self.http_resp = responses.http_resp
         self.https_resp = responses.https_resp
@@ -70,6 +72,69 @@ class DEFPAGE:
         except socket.gaierror:
             return None
 
+    def _test_invalid_host_header(self, protocol: str) -> Optional[object]:
+        """
+        Test server response with invalid Host header.
+        
+        Tests if server returns the same page when accessing with invalid Host header.
+        
+        Args:
+            protocol: 'http' or 'https'
+            
+        Returns:
+            Response object or None if request failed
+        """
+        base_url = f"{protocol}://{self.target_ip}"
+        
+        try:            
+            response = self.helpers._raw_request(base_url, "/", extra_headers={"Host": "%"})
+            
+            if response:                
+                return response
+            
+        except Exception as e:
+            return None
+        return None
+
+    def _compare_responses(self, resp1: object, resp2: object, protocol: str) -> Dict[str, Any]:
+        """
+        Compare two responses to check if they are the same.
+        
+        Args:
+            resp1: First response object (IP access)
+            resp2: Second response object (invalid Host header)
+            protocol: Protocol used
+            
+        Returns:
+            Dictionary with comparison results
+        """
+        result = {
+            'same_status': False,
+            'same_content': False,
+            'responses_match': False,
+            'status1': None,
+            'status2': None
+            }
+        
+        if not resp1 or not resp2:
+            return result
+        
+        status1 = getattr(resp1, 'status', None) or getattr(resp1, 'status_code', None)
+        status2 = getattr(resp2, 'status', None) or getattr(resp2, 'status_code', None)
+        
+        result['status1'] = status1
+        result['status2'] = status2
+        result['same_status'] = (status1 == status2)
+        
+        content1 = getattr(resp1, 'text', '') or ''
+        content2 = getattr(resp2, 'text', '') or ''
+        
+        if content1 and content2:
+            result['same_content'] = (content1 == content2)
+        
+        result['responses_match'] = result['same_status'] and result['same_content']
+        return result
+
     def run(self) -> None:
         """
         Execute the DEFPAGE test logic.
@@ -125,34 +190,83 @@ class DEFPAGE:
                 self._debug_output(content, protocol, response, url)
 
             self.default_page_reachable = True
-            technologies = self._analyze_page_content(content, protocol, response)
             
-            if technologies:
-                if self.args.verbose:
-                    ptprint("Technologies detected:", "ADDITIONS", not self.args.json, indent=8, colortext=True)
-
-                for tech in technologies:
-                    tech['protocol'] = protocol
-                    tech['url'] = url
-                    self.detected_technologies.append(tech)
-                    
-                    version_text = f" {tech['version']}" if tech.get('version') else ""
-                    category_text = f" ({tech['category']})"
-                    
+            invalid_host_response = self._test_invalid_host_header(protocol)
+            
+            if invalid_host_response:
+                invalid_status_code = getattr(invalid_host_response, 'status', 0) or getattr(invalid_host_response, 'status_code', 0)
+                
+                if invalid_status_code == 200 or invalid_status_code == 403:
+                    comparison = self._compare_responses(response, invalid_host_response, protocol)
+                                        
+                    # If responses are different, analyze both
+                    if not comparison['responses_match']:
+                        if self.args.verbose:
+                            ptprint("Responses differ - analyzing both pages", "ADDITIONS", not self.args.json, indent=8, colortext=True)
+                        
+                        technologies = self._analyze_page_content(content, protocol, response)
+                        self._process_detected_technologies(technologies, protocol, url, "IP access")
+                        
+                        invalid_content = getattr(invalid_host_response, 'text', '')
+                        if invalid_content:
+                            invalid_technologies = self._analyze_page_content(invalid_content, protocol, invalid_host_response)
+                            self._process_detected_technologies(invalid_technologies, protocol, url, "invalid Host header")
+                    else:
+                        if self.args.verbose:
+                            ptprint("Responses match - analyzing once", "ADDITIONS", not self.args.json, indent=8, colortext=True)
+                        
+                        #If responses are the same, analyze once
+                        technologies = self._analyze_page_content(content, protocol, response)
+                        self._process_detected_technologies(technologies, protocol, url, "both methods")
+                else:
+                    #If Invalid Host response has different status code, analyze only original
                     if self.args.verbose:
-                        ptprint(f"{tech['technology']}{version_text}{category_text}", 
-                        "ADDITIONS", not self.args.json, indent=12, colortext=True, end="")
-
-                        source_location = tech.get('source_location', 'content')
-                        ptprint(f" <- Matched: {tech.get('matched_text', 'N/A')} (from {source_location})", 
-                            "ADDITIONS", not self.args.json, colortext=True)
+                        ptprint(f"Invalid Host header returned HTTP {invalid_status_code} - skipping analysis", "ADDITIONS", not self.args.json, indent=8, colortext=True)
+                    
+                    technologies = self._analyze_page_content(content, protocol, response)
+                    self._process_detected_technologies(technologies, protocol, url, "IP access")
             else:
-                if self.args.verbose:
-                    ptprint(f"No technologies detected", "ADDITIONS", not self.args.json, indent=8, colortext=True)
+                technologies = self._analyze_page_content(content, protocol, response)
+                self._process_detected_technologies(technologies, protocol, url, "IP access")
+                
         else:
             if self.args.verbose:
                 ptprint(f"Default page of server is not reachable (HTTP {status_code})", "ADDITIONS", not self.args.json, indent=8, colortext=True)
 
+    def _process_detected_technologies(self, technologies: List[Dict[str, Any]], protocol: str, url: str, access_method: str) -> None:
+        """
+        Process and report detected technologies.
+        
+        Args:
+            technologies: List of detected technologies
+            protocol: Protocol used
+            url: URL tested
+            access_method: How the page was accessed (e.g., "IP access", "invalid Host header")
+        """
+        if technologies:
+            if self.args.verbose:
+                ptprint(f"Technologies detected ({access_method}):", "ADDITIONS", not self.args.json, indent=8, colortext=True)
+
+            for tech in technologies:
+                tech['protocol'] = protocol
+                tech['url'] = url
+                tech['access_method'] = access_method
+                self.detected_technologies.append(tech)
+                
+                version_text = f" {tech['version']}" if tech.get('version') else ""
+                category_text = f" ({tech['category']})"
+                
+                if self.args.verbose:
+                    ptprint(f"{tech['technology']}{version_text}{category_text}", 
+                    "ADDITIONS", not self.args.json, indent=12, colortext=True, end="")
+
+                    source_location = tech.get('source_location', 'content')
+                    ptprint(f" <- Matched: {tech.get('matched_text', 'N/A')}", 
+                        "ADDITIONS", not self.args.json, colortext=True)
+        else:
+            if self.args.verbose:
+                ptprint(f"No technologies detected ({access_method})", "ADDITIONS", not self.args.json, indent=8, colortext=True)
+
     def _debug_output(self, content: str, protocol: str, response: object, url: str) -> None:
         """
         Debug output shown when the -vv flag is used.
@@ -295,6 +409,9 @@ class DEFPAGE:
                     if pattern_def.get("submodule"):
                         match_result = self._call_submodule(match_result, pattern_def["submodule"], response, content)
                     
+                    # Determine version range based on technology
+                    match_result = self._determine_version_range(match_result, content)
+                    
                     detected.append(match_result)
                     seen_technologies.add(tech_key)
         
@@ -329,16 +446,34 @@ class DEFPAGE:
         try:
             match = re.search(pattern, content, re_flags)
         except re.error as e:
-            ptprint(f"Invalid regex pattern in definitions: {e}", "INFO", not self.args.json, indent=8)
             return None
         
         if not match:
             return None
+        
+        # Get product information from product_id
+        product_id = pattern_def.get('product_id')
+        if product_id:
+            product = self.product_manager.get_product_by_id(product_id)
+            if product:
+                technology_name = product.get('our_name', 'Unknown')
+                category_id = product.get('category_id')
+                category = self.product_manager.get_category_by_id(category_id)
+                category_name = category.get('name', 'Other') if category else 'Other'
+            else:
+                # Fallback if product not found
+                technology_name = pattern_def.get('name', 'Unknown')
+                category_name = 'Other'
+        else:
+            # Backward compatibility with old format
+            technology_name = pattern_def.get('technology', pattern_def.get('name', 'Unknown'))
+            category_name = pattern_def.get('category', 'Other')
             
         result = {
             'name': pattern_def.get('name', 'Unknown'),
-            'category': pattern_def.get('category', 'unknown'),
-            'technology': pattern_def.get('technology', pattern_def.get('name', 'Unknown')),
+            'category': category_name,
+            'technology': technology_name,
+            'product_id': product_id,
             'version': None,
             'matched_text': match.group(0)[:100] + ('...' if len(match.group(0)) > 100 else ''),
             'source_location': source_type
@@ -365,6 +500,145 @@ class DEFPAGE:
         
         return result
 
+    def _determine_version_range(self, tech_info: Dict[str, Any], content: str) -> Dict[str, Any]:
+        """
+        Determine version range for detected technology based on structural characteristics.
+        Uses rules defined in JSON configuration file.
+        
+        Args:
+            tech_info: Technology information dictionary
+            content: Page content
+            
+        Returns:
+            Enhanced technology information with version range
+        """
+        technology = tech_info.get('technology', '').lower()
+        
+        version_detection = self.definitions.get('version_detection', {})
+        
+        matched_tech_key = None
+        for tech_key, tech_data in version_detection.items():
+            aliases = tech_data.get('technology_aliases', [])
+            for alias in aliases:
+                if alias.lower() in technology:
+                    matched_tech_key = tech_key
+                    break
+            if matched_tech_key:
+                break
+        
+        if not matched_tech_key:
+            return tech_info
+        
+        return self._apply_version_rules(tech_info, content, version_detection[matched_tech_key])
+
+    def _apply_version_rules(self, tech_info: Dict[str, Any], content: str, rules: Dict[str, Any]) -> Dict[str, Any]:
+        """
+        Apply version detection rules from JSON configuration.
+        
+        Args:
+            tech_info: Technology information dictionary
+            content: Page content
+            rules: Version detection rules for specific technology
+            
+        Returns:
+            Enhanced technology information with version range
+        """
+        # Use product_id from version_detection rules if available
+        if 'product_id' in rules and 'product_id' not in tech_info:
+            tech_info['product_id'] = rules['product_id']
+        
+        ranges = rules.get('ranges', [])
+        best_match = None
+        best_score = 0
+        best_confidence = 'low'
+        
+        for range_def in ranges:
+            indicators = range_def.get('indicators', [])
+            required_matches = range_def.get('required_matches', 1)
+            specific_indicator = range_def.get('specific_indicator')
+            confidence = range_def.get('confidence', 'low')
+            
+            score = 0
+            for pattern in indicators:
+                try:
+                    if re.search(pattern, content, re.I | re.S):
+                        score += 1
+                except re.error as e:
+                    if self.args.verbose:
+                        ptprint(f"Invalid regex pattern in version detection: {pattern} - {e}", 
+                            "INFO", not self.args.json, indent=8)
+                    continue
+            
+            specific_ok = True
+            if specific_indicator:
+                try:
+                    specific_ok = bool(re.search(specific_indicator, content, re.I | re.S))
+                except re.error as e:
+                    if self.args.verbose:
+                        ptprint(f"Invalid specific indicator regex: {specific_indicator} - {e}", 
+                            "INFO", not self.args.json, indent=8)
+                    specific_ok = False
+            
+            if score >= required_matches and specific_ok:
+                if score > best_score or (score == best_score and self._compare_confidence(confidence, best_confidence) > 0):
+                    best_score = score
+                    best_match = range_def
+                    best_confidence = confidence
+        
+        if best_match:
+            version_range_info = {
+                'min_version': best_match.get('min_version'),
+                'max_version': best_match.get('max_version'),
+                'confidence': best_confidence
+            }
+            
+            tech_info['version_range'] = version_range_info
+        
+        return tech_info
+
+    def _compare_confidence(self, conf1: str, conf2: str) -> int:
+        """
+        Compare two confidence levels.
+        
+        Args:
+            conf1: First confidence level ('low', 'medium', 'high')
+            conf2: Second confidence level ('low', 'medium', 'high')
+            
+        Returns:
+            1 if conf1 > conf2, -1 if conf1 < conf2, 0 if equal
+        """
+        confidence_order = {'low': 0, 'medium': 1, 'high': 2}
+        level1 = confidence_order.get(conf1.lower(), 0)
+        level2 = confidence_order.get(conf2.lower(), 0)
+        
+        if level1 > level2:
+            return 1
+        elif level1 < level2:
+            return -1
+        else:
+            return 0
+
+
+    def _format_version_range(self, min_version: Optional[str], max_version: Optional[str]) -> str:
+        """
+        Format version range for display.
+        
+        Args:
+            min_version: Minimum version
+            max_version: Maximum version
+            
+        Returns:
+            Formatted version range string
+        """
+        if min_version and max_version:
+            return f"{min_version} - {max_version}"
+        elif min_version:
+            return f"{min_version}+"
+        elif max_version:
+            return f"< {max_version}"
+        else:
+            return "unknown"
+
     def _call_submodule(self, tech_info: Dict[str, Any], submodule_name: str, response: object, content: str) -> Dict[str, Any]:
         """
         Calls specified submodule for enhanced technology detection.
@@ -386,7 +660,7 @@ class DEFPAGE:
                 enhanced_tech_info['response'] = response
                 enhanced_tech_info['content'] = content
                 
-                enhanced_info = submodule.analyze(enhanced_tech_info, self.args)
+                enhanced_info = submodule.analyze(enhanced_tech_info, self.args, self.helpers)
                 tech_info.update(enhanced_info)
                                     
         except ImportError as e:
@@ -421,18 +695,17 @@ class DEFPAGE:
         """
         if not self.detected_technologies:
             if self.default_page_reachable:
-                ptprint("Overall Summary: No default page technologies identified", "INFO", not self.args.json, indent=4)
+                ptprint("No default page technologies identified", "INFO", not self.args.json, indent=4)
             else:
-                ptprint("Overall Summary: Default page of server is not reachable", "INFO", not self.args.json, indent=4)
+                ptprint("Default page of server is not reachable", "INFO", not self.args.json, indent=4)
             return
-
-        ptprint("Overall Technology Summary", "INFO", not self.args.json, indent=4, colortext=True)
         
         tech_summary = {}
         for tech in self.detected_technologies:
             tech_name = tech.get('technology', 'Unknown')
             version = tech.get('version', '')
             protocol = tech.get('protocol', 'unknown')
+            probability = tech.get('probability', 100)
             
             key = f"{tech_name}_{version}" if version else tech_name
             
@@ -443,7 +716,9 @@ class DEFPAGE:
                     'category': tech.get('category', 'unknown'),
                     'protocols': [],
                     'source_locations': set(),
-                    'additional_info': tech.get('additional_info', [])  # Added for submodule data
+                    'additional_info': tech.get('additional_info', []),
+                    'version_range': tech.get('version_range', {}),
+                    'product_id': tech.get('product_id')
                 }
             
             tech_summary[key]['protocols'].append(protocol.upper())
@@ -457,21 +732,27 @@ class DEFPAGE:
                         existing_info.append(info)
          
         for tech_info in tech_summary.values():
-            version_text = f" {tech_info['version']}" if tech_info['version'] else ""
             protocols_text = "/".join(sorted(set(tech_info['protocols'])))
             category_text = f" ({tech_info['category']})"
             
-            ptprint(f"{tech_info['name']}{version_text}{category_text}", "VULN", not self.args.json, indent=8, end=" ")
+            ptprint(f"{tech_info['name']}{category_text}", "VULN", not self.args.json, indent=4, end=" ")
+            ptprint(f"({probability}%)", "ADDITIONS", not self.args.json, colortext=True)
 
-            if self.args.verbose:
-                ptprint(f"(detected via {protocols_text})", "ADDITIONS", not self.args.json, colortext=True)
-            else:
-                ptprint(" ")
+            if tech_info['version']:
+                ptprint(f"Version: {tech_info['version']}", "INFO", not self.args.json, indent=12)
+            elif tech_info.get('version_range'):
+                version_range = tech_info['version_range']
+                range_text = self._format_version_range(
+                    version_range.get('min_version'),
+                    version_range.get('max_version')
+                )
+                ptprint(f"Version range: {range_text}", "INFO", not self.args.json, indent=8)
+            
                 
-            # Report additional info from submodules
+            # Report additional info from submodules and version ranges
             if tech_info.get("additional_info"):
                 for info in tech_info["additional_info"]:
-                    ptprint(f"{info}", "INFO", not self.args.json, indent=12)
+                    ptprint(f"{info}", "VULN", not self.args.json, indent=8)
 
         for tech_info in tech_summary.values():
             self._store_unique_technology(tech_info)
@@ -487,21 +768,53 @@ class DEFPAGE:
         version = tech_info['version']
         tech_type = tech_info['category']
         probability = 100
+        product_id = tech_info.get('product_id')
+
+        version_range = tech_info.get('version_range', {})
+        if version_range:
+            version_min = version_range.get('min_version')
+            version_max = version_range.get('max_version')
+        else:
+            version_min = None
+            version_max = None
         
+        # Get product info for vendor and CVE details URL
+        vendor = None
+        cve_details_url = None
+        if product_id:
+            product = self.product_manager.get_product_by_id(product_id)
+            if product:
+                vendor = product.get('vendor')
+                cve_details_url = product.get('cve_details_url')
+            
         protocols_text = "/".join(sorted(set(tech_info['protocols'])))
         source_locations = sorted(tech_info['source_locations'])
         
         description = f"Default {protocols_text} page: {tech_name}"
         if version:
             description += f" {version}"
+        
+        version_range = tech_info.get('version_range', {})
+        if version_range.get('min_version') or version_range.get('max_version'):
+            range_text = self._format_version_range(
+                version_range.get('min_version'),
+                version_range.get('max_version')
+            )
+            description += f" (range: {range_text})"
+        
         description += f" (detected from {', '.join(source_locations)})"
         
         storage.add_to_storage(
             technology=tech_name,
-            version=version,
+            version=version if version else None,
+            version_min=version_min,
+            version_max=version_max,
             technology_type=tech_type,
             probability=probability,
-            description=description
+            description=description,
+            product_id=product_id,
+            vendor=vendor,
+            cve_details_url=cve_details_url
         )
 
 
