@@ -19,6 +19,7 @@ from ptlibs import ptjsonlib
 from ptlibs.ptprinthelper import ptprint
 
 from helpers.stored_responses import StoredResponses
+from helpers.products import get_product_manager
 from helpers.result_storage import storage
 
 import json
@@ -38,6 +39,7 @@ class WSURLLEN:
         self.ptjsonlib = ptjsonlib
         self.helpers = helpers
         self.http_client = http_client
+        self.product_manager = get_product_manager()
 
         # Unpack stored responses
         self.response_hp = responses.resp_hp
@@ -96,10 +98,17 @@ class WSURLLEN:
         if blocked_long_url:
             ptprint("Long URL are blocked", "INFO", not self.args.json,indent=4)
 
-        server = self._identify_server(statuses)
+        server, probability, product_id = self._identify_server(statuses)
         if server:
-            ptprint(f"Identified WS: {server}", "VULN", not self.args.json, indent=4)
-            storage.add_to_storage(technology=server, technology_type="WebServer", vulnerability="PTV-WEB-INFO-WSURL", probability=20)
+            ptprint(f"Identified WS: {server} ", "VULN", not self.args.json, indent=4, end="")
+            ptprint(f"({probability}%)", "ADDITIONS", not self.args.json, colortext=True)
+            storage.add_to_storage(
+                technology=server, 
+                technology_type="Web Server", 
+                vulnerability="PTV-WEB-INFO-WSURL", 
+                probability=probability,
+                product_id=product_id
+            )
         else:
             ptprint("No matching web server identified from URL length behavior", "INFO", not self.args.json, indent=4)
 
@@ -111,12 +120,24 @@ class WSURLLEN:
             observed_statuses: List of HTTP status codes for each tested URL length.
 
         Returns:
-            Detected server name if match is found, otherwise None.
+            Tuple of (technology_name, probability, product_id) if match found, otherwise (None, None, None).
         """
         for entry in self.definitions:
             if entry.get("statuses") == observed_statuses:
-                return entry.get("technology")
-        return None
+                # Get product info from product_id
+                product_id = entry.get("product_id")
+                if not product_id:
+                    continue
+                
+                product = self.product_manager.get_product_by_id(product_id)
+                if not product:
+                    continue
+                
+                technology_name = product.get("our_name", "Unknown")
+                probability = entry.get("probability", 100)
+                
+                return technology_name, probability, product_id
+        return None, None, None
 
 
 def run(args: object, ptjsonlib: object, helpers: object, http_client: object, responses: StoredResponses):
